"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSocialTokenMetadata = exports.useTwitterTokenRef = exports.useClaimedTokenRef = exports.useMintTokenRef = exports.useTokenRefFromBonding = exports.useClaimedTokenRefKey = exports.useClaimedTwitterTokenRefKey = exports.useUnclaimedTwitterTokenRefKey = exports.getTwitterUnclaimedTokenRefKey = exports.getTwitterClaimedTokenRefKey = exports.WUMBO_TWITTER_TLD = void 0;
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const web3_js_1 = require("@solana/web3.js");
const spl_token_collective_1 = require("@strata-foundation/spl-token-collective");
const hooks_1 = require("../hooks");
const react_1 = require("react");
const react_async_hook_1 = require("react-async-hook");
const nameServiceTwitter_1 = require("../utils/nameServiceTwitter");
const useTokenMetadata_1 = require("./useTokenMetadata");
exports.WUMBO_TWITTER_TLD = new web3_js_1.PublicKey("Fhqd3ostRQQE65hzoA7xFMgT9kge2qPnsTNAKuL2yrnx");
function getTwitterClaimedTokenRefKey(connection, handle, mint = undefined, tld = exports.WUMBO_TWITTER_TLD) {
    return __awaiter(this, void 0, void 0, function* () {
        const owner = (yield (0, nameServiceTwitter_1.getTwitterRegistry)(connection, handle, tld)).owner;
        return (yield spl_token_collective_1.SplTokenCollective.ownerTokenRefKey({
            owner,
            mint,
        }))[0];
    });
}
exports.getTwitterClaimedTokenRefKey = getTwitterClaimedTokenRefKey;
function getTwitterUnclaimedTokenRefKey(handle, mint = spl_token_collective_1.SplTokenCollective.OPEN_COLLECTIVE_MINT_ID, tld = exports.WUMBO_TWITTER_TLD) {
    return __awaiter(this, void 0, void 0, function* () {
        const name = yield (0, nameServiceTwitter_1.getTwitterRegistryKey)(handle, tld);
        return (yield spl_token_collective_1.SplTokenCollective.ownerTokenRefKey({
            name,
            mint,
        }))[0];
    });
}
exports.getTwitterUnclaimedTokenRefKey = getTwitterUnclaimedTokenRefKey;
const useUnclaimedTwitterTokenRefKey = (name, collective = spl_token_collective_1.SplTokenCollective.OPEN_COLLECTIVE_ID, tld = exports.WUMBO_TWITTER_TLD) => {
    const { connection } = (0, wallet_adapter_react_1.useConnection)();
    const { result: key, loading } = (0, react_async_hook_1.useAsync)((name, collective) => __awaiter(void 0, void 0, void 0, function* () {
        if (connection && name) {
            return getTwitterUnclaimedTokenRefKey(name, collective, tld);
        }
    }), [name, collective]);
    return { result: key, loading };
};
exports.useUnclaimedTwitterTokenRefKey = useUnclaimedTwitterTokenRefKey;
const useClaimedTwitterTokenRefKey = (name, collective = web3_js_1.PublicKey.default, tld = exports.WUMBO_TWITTER_TLD) => {
    const { connection } = (0, wallet_adapter_react_1.useConnection)();
    const { result: key, loading } = (0, react_async_hook_1.useAsync)((connection, name, collective, tld) => __awaiter(void 0, void 0, void 0, function* () {
        if (connection && name) {
            return getTwitterClaimedTokenRefKey(connection, name, collective, tld);
        }
    }), [connection, name, collective, tld]);
    return { result: key, loading };
};
exports.useClaimedTwitterTokenRefKey = useClaimedTwitterTokenRefKey;
const useClaimedTokenRefKey = (owner, mint = web3_js_1.PublicKey.default) => {
    const { result } = (0, react_async_hook_1.useAsync)((owner) => __awaiter(void 0, void 0, void 0, function* () { return owner && spl_token_collective_1.SplTokenCollective.ownerTokenRefKey({ owner, mint }); }), [owner]);
    return result ? result[0] : undefined;
};
exports.useClaimedTokenRefKey = useClaimedTokenRefKey;
/**
 * Get a token ref from the bonding instance
 *
 * @param tokenBonding
 * @returns
 */
function useTokenRefFromBonding(tokenBonding) {
    const bonding = (0, hooks_1.useTokenBonding)(tokenBonding);
    const { result: key } = (0, react_async_hook_1.useAsync)((bonding) => __awaiter(this, void 0, void 0, function* () { return bonding && spl_token_collective_1.SplTokenCollective.mintTokenRefKey(bonding.targetMint); }), [bonding.info]);
    return (0, hooks_1.useTokenRef)(key && key[0]);
}
exports.useTokenRefFromBonding = useTokenRefFromBonding;
/**
 * Given a social token mint, get the social token TokenRef
 *
 * @param mint
 * @returns
 */
function useMintTokenRef(mint) {
    const { result: key } = (0, react_async_hook_1.useAsync)((mint) => __awaiter(this, void 0, void 0, function* () { return mint && spl_token_collective_1.SplTokenCollective.mintTokenRefKey(mint); }), [mint]);
    return (0, hooks_1.useTokenRef)(key && key[0]);
}
exports.useMintTokenRef = useMintTokenRef;
/**
 * Get the token ref for this wallet
 * @param owner
 * @returns
 */
function useClaimedTokenRef(owner) {
    const key = (0, exports.useClaimedTokenRefKey)(owner);
    return (0, hooks_1.useTokenRef)(key);
}
exports.useClaimedTokenRef = useClaimedTokenRef;
/**
 * Get a TokenRef using a twitter handle name service lookup on `name`. Searches for `name`, then grabs the owner.
 *
 * If the name is unclaimed, grabs the unclaimed token ref if it exists
 *
 * @param name
 * @param collective
 * @param tld
 * @returns
 */
const useTwitterTokenRef = (name, collective, tld = exports.WUMBO_TWITTER_TLD) => {
    const { result: claimedKey, loading: twitterLoading } = (0, exports.useClaimedTwitterTokenRefKey)(name, tld);
    const { result: unclaimedKey, loading: claimedLoading } = (0, exports.useUnclaimedTwitterTokenRefKey)(name, collective, tld);
    const claimed = (0, hooks_1.useTokenRef)(claimedKey);
    const unclaimed = (0, hooks_1.useTokenRef)(unclaimedKey);
    const result = (0, react_1.useMemo)(() => {
        if (claimed.info) {
            return claimed;
        }
        return unclaimed;
    }, [claimed === null || claimed === void 0 ? void 0 : claimed.info, unclaimed === null || unclaimed === void 0 ? void 0 : unclaimed.info, claimed.loading, unclaimed.loading]);
    const loading = (0, react_1.useMemo)(() => {
        return (twitterLoading ||
            claimedLoading ||
            !unclaimedKey ||
            claimed.loading ||
            unclaimed.loading);
    }, [
        twitterLoading,
        claimedLoading,
        name,
        claimedKey,
        unclaimedKey,
        claimed,
        unclaimed,
    ]);
    return Object.assign(Object.assign({}, result), { loading });
};
exports.useTwitterTokenRef = useTwitterTokenRef;
/**
 * Get all metadata associated with a given wallet's social token.
 *
 * @param owner
 * @returns
 */
function useSocialTokenMetadata(owner) {
    const { info: tokenRef, loading } = useClaimedTokenRef(owner);
    const { info: tokenBonding } = (0, hooks_1.useTokenBonding)((tokenRef === null || tokenRef === void 0 ? void 0 : tokenRef.tokenBonding) || undefined);
    return Object.assign(Object.assign({}, (0, useTokenMetadata_1.useTokenMetadata)(tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.targetMint)), { tokenRef,
        tokenBonding });
}
exports.useSocialTokenMetadata = useSocialTokenMetadata;
//# sourceMappingURL=tokenRef.js.map